/*
 * Copyright (C) 2003-2006 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
//
// @author Stefano Fornari
// @version $Id: SPDMUtils.cpp,v 1.2 2004/02/09 08:48:50 ste Exp $
//

#include "spdm/common/Utils.h"


USING_PTYPES
 /*
 * Extracts the node name from the node path
 *
 * @param node - the node path
 * @param name - the buffer that will contain the node name
 * @param size - buffer size
 */
void getNodeName(const wchar_t* node, wchar_t* name, int size) {
    wchar_t* p;

    p = wcsrchr(node, L'/');

    if (p == NULL) {
        wcsncpy(name, node, size-1);
        return;
    }

    wcsncpy(name, p+1, size);
}

/*
 * Extracts the node context from the node path
 *
 * @param node - the node path
 * @param context - the buffer that will contain the node context
 * @param size - buffer size
 */
void getNodeContext(const wchar_t* node, wchar_t* context, int size) {
    wchar_t* p;

    p = wcsrchr(node, L'/');

    if (p == NULL) {
        *context = L'\0';
        return;

    }

    wcsncpy(context, node, min((p-node), size));
}


/*
* Function that convert char* to wchar_t*. It creates buffer with new operator
* and has to be descarded by the caller
*/

wchar_t* char2wchar(char *token) {
    
	int len = 0;
    wchar_t *ret = NULL;	
    if (token == NULL) 
        goto finally;
    
    len = strlen(token);
    
    if (len == 0)
        goto finally;

	ret = new wchar_t[len + 1];
	MultiByteToWideChar (CP_ACP, 0, token, len, ret, len);
	ret[len] = 0;	

finally:
    return ret;
}

/*
* Function that convert wchar_t* to char*. It creates buffer with new operator
* and has to be descarded by the caller
*/

char* wchar2char(wchar_t* wtoken) {

    int len = 0;
	BOOL usedDefaultChar;
    char* ret = NULL;
    if (wtoken == NULL)
        goto finally;
    
    len = wcslen(wtoken);    
    if (len == 0)
        goto finally;

    ret = new char[len + 1];
		
	WideCharToMultiByte (CP_ACP, 0, wtoken, len, ret, len, "?", &usedDefaultChar);		
	ret[len] = 0;
    

finally:
    return ret;

}


/*
* Function used to remove xmlns="syncml:metinf" or xmlns="syncml:SYNCML1.1" from tags of xml msg.
* suppose tag with name space is like <Type xmlns="syncml:metinf"> or <SyncML xmlns="syncml:SYNCML1.1">
*/
/*
wchar_t* removeNameSpace(wchar_t* wtoken) {
    
    wchar_t* ret = NULL;
    wchar_t* ptrFound = NULL;
    int ptrBeforeFound = 0;
    wchar_t* ptrStart = NULL;
    wchar_t* SYNCML_METAINF = TEXT(" xmlns=\"syncml:metinf\"");
    
    
    int len = 0;
    


    if (wtoken == NULL)
        goto finally;

    len = wcslen(wtoken);
    if (len == 0)
        goto finally;
    ret = new wchar_t[len + 1];
    ret[len] = 0;
    
    ptrStart = wtoken;
    
    do {
        ptrFound = NULL;
        ptrFound = wcsstr(ptrStart, SYNCML_METAINF);
        if (ptrFound == NULL)
            break;

        wmemcpy(&ret[ptrBeforeFound], ptrStart, ptrFound - ptrStart);
        
        ptrBeforeFound = ptrFound - ptrStart;

        ptrStart = ptrFound + wcslen(SYNCML_METAINF);
        
    
    } while(ptrFound != NULL);


finally:
    return ret;   

}
*/

wchar_t* removeNameSpace(wchar_t* wtoken) {
    
    wchar_t* ret = NULL;
    int position = 0;
    wchar_t* SYNCML_METAINF = TEXT(" xmlns=\"syncml:metinf\"");

    string s = string();
    s += wtoken;
    
    position = pos(SYNCML_METAINF, s);
    while (position != -1) {
        del(s, position, wcslen(SYNCML_METAINF));
        //ins(TEXT("<"), s, position);
        position = pos(SYNCML_METAINF, s);
    }
    
    // ret = new wchar_t[wcslen(wtoken) + 1];
    wcscpy(wtoken,(wchar_t*)s);
    wtoken[length(s)] = 0;
    return ret;
}