/*
 * Copyright (C) 2003-2006 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

//
// @author Marco Magistrali
// @version $Id: SyncMLDMConfig.cpp,v 1.0 2004/04/27 11:21:00 magi Exp $
//

#include "spdm/win32/SyncMLDMConfig.h"
#include "common/Log.h"
#include "common/Constants.h"
#include "spdm/win32/Win32ManagementTree.h"



SyncMLDMConfig::SyncMLDMConfig(ManagementNode* node, wchar_t* accNodeName) {
    LOG.debug(TEXT("in SyncMLDMConfig"));
   
    wchar_t buf[DIM_BUFFER_PATH];
    wchar_t* childrenName = NULL;

    syncMLDMAcc = NULL;
    syncMLDMCon = NULL;

    // set name of accountName
    nameDMAcc = accNodeName;

    Win32ManagementTree mt = Win32ManagementTree();
    ManagementNode* nodeCon = NULL;

    node->getFullName(buf, DIM_BUFFER_PATH);
    
    //
    // if accNodeName is null, try to use the first found account...
    //

    if (accNodeName == NULL) { 
        
        childrenName = new wchar_t[DIM_BUFFER_PATH];
        wchar_t bufTmp [DIM_BUFFER_PATH];
        wchar_t *childName = NULL;
        wsprintf(bufTmp, TEXT("%s/%s"), buf, DM_ACC_CONTEXT);

        nodeCon = mt.getManagementNode(bufTmp);
        if (nodeCon != NULL) {
            wmemset(childrenName, 0, DIM_BUFFER_PATH);
            nodeCon->getChildrenName(bufTmp, childrenName, DIM_BUFFER_PATH, FALSE);
            
            childName = wcschr(childrenName, L'/');
            if (wcslen(childrenName) != 0) {
                if (childName == NULL) 
                    nameDMAcc = childrenName;
                else {
                    *childName = 0;
                    nameDMAcc = childrenName;                   
                }
            }
            else {
                LOG.error(TEXT("NO DMAcc found!!"));   
                goto finally;
            }           
                        
        }  else {
            LOG.error(TEXT("NO DMAcc node found!!"));   
            goto finally;
        }
    }

    wsprintf(buf, TEXT("%s/%s/%s"), buf, DM_ACC_CONTEXT, nameDMAcc);
        
    nodeCon = mt.getManagementNode(buf);
    
    if (nodeCon == NULL) {    
        lastErrorCode = ERR_INVALID_CONTEXT;
        wsprintf(lastErrorMsg, TEXT("DMAcc node configuration not found"));
        LOG.error(lastErrorMsg);
        goto finally;
    }
   
    syncMLDMAcc = new SyncMLDMAcc(nodeCon);
    delete nodeCon; nodeCon = NULL;

    if (lastErrorCode != 0) {
        LOG.info(TEXT("syncMLDMAcc NULL"));
        goto finally;
    }
    
    node->getFullName(buf, DIM_BUFFER_PATH);
    wsprintf(buf, TEXT("%s/%s"), buf, DM_CON_CONTEXT);    
        
    nodeCon = mt.getManagementNode(buf);  

    if (nodeCon == NULL) {    
        lastErrorCode = ERR_INVALID_CONTEXT;
        wsprintf(lastErrorMsg, TEXT("DevInfo configuration not found"));
        LOG.error(lastErrorMsg);
        goto finally;
    }

    syncMLDMCon = new SyncMLDMCon(nodeCon);
    delete nodeCon; nodeCon = NULL;

    if (lastErrorCode != 0) {
        LOG.info(TEXT("syncMLDMCon NULL"));
        goto finally;
    }   

finally:
    return;

}


/*
 * Destructor
 */
SyncMLDMConfig::~SyncMLDMConfig() {

    if (syncMLDMAcc != NULL) {
        delete syncMLDMAcc; syncMLDMAcc = NULL;
    }

    if (syncMLDMCon != NULL) {
        delete syncMLDMCon; syncMLDMCon = NULL;
    }
}


BOOL SyncMLDMConfig::saveToDM() {
    
    Win32ManagementTree mt = Win32ManagementTree();
    
    if (syncMLDMAcc != NULL)
        syncMLDMAcc->saveToDM(&mt, nameDMAcc);
    
    //
    // Todo: saveToDM of syncMLDMCon
    //

    return TRUE;
}