/*
 * Copyright (C) 2003-2006 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

//
// @author Marco Magistrali
// @version $Id: SyncMLDMProcessor.cpp,v 1.0 2004/04/27 11:21:00 magi Exp $
//



#include "spdm/win32/SyncMLDMProcessor.h"
#include "spdm/common/Utils.h"
#include "spdm/common/Constants.h"
#include "spds/common/Utils.h"
#include "spdm/win32/Win32ManagementTree.h"
#include "spdm/common/ManagementNode.h"


USING_PTYPES

SyncMLDMProcessor::SyncMLDMProcessor() {
    return;
}


/*
 * Processes the initialization response. Returns 0 in case of success, an
 * error code in case of error.
 *
 * @param msg the response from the server
 */
int SyncMLDMProcessor::processInitResponse(wchar_t* msg, wchar_t** storeNextNonce) {
    wchar_t* syncBody = NULL;
    wchar_t* alert    = NULL;
    wchar_t* item     = NULL;
    wchar_t* target   = NULL;
    wchar_t* uri      = NULL;
    wchar_t* data     = NULL;
    //unsigned int n, pos;
    int ret = -1;

    //
    // We have to look for the Alert element relative to the given source.
    // We will store the alert code in the SyncSource object.
    // If no valid Alert is found, we set lastErrorCode and lastErrorMsg
    // accordingly.
    //
    syncBody = getElementContent(msg, TEXT("SyncBody"), NULL);
    if (syncBody == NULL) {
        ret = lastErrorCode = ERR_REPRESENTATION;
        wsprintf(lastErrorMsg, TEXT("SyncBody not found!"));

        goto finally;
    }

    //
    // First of all check the status for the SyncHead
    //
    ret = getSyncHeaderStatusCode(syncBody, storeNextNonce);
    if ((ret <= 200) || (ret >=299)) {
        goto finally;
    }

    //
    // If the access is allowed
    ret = getAlertAndReplaceStatusCode(syncBody);
    if ((ret < 200) || (ret > 299)) {
        goto finally;
    }

    //
    // Finally ...
    //
finally:

    safeDelete(&syncBody);
    safeDelete(&alert   );
    safeDelete(&item    );
    safeDelete(&target  );
    safeDelete(&uri     );
    safeDelete(&data    );

    return ret;
}

/****************************************************************/
int SyncMLDMProcessor::getServerMessageId(wchar_t* msg) {
    
    wchar_t* element = NULL;
    int ret = 1;

    element = getElementContent(msg, TEXT("MsgID"), NULL);

    if (element != NULL) 
        ret = wcstoul(element, NULL, 10);
    
    safeDelete (&element);
    return ret;
}


int SyncMLDMProcessor::processHeader(wchar_t* msg) {
    
    wchar_t* syncBody       = NULL;
    wchar_t* storeNextNonce = NULL;
    int ret = -1;

    syncBody = getElementContent(msg, TEXT("SyncBody"), NULL);
    if (syncBody == NULL) {
        ret = lastErrorCode = ERR_REPRESENTATION;
        wsprintf(lastErrorMsg, TEXT("SyncBody not found!"));
        goto finally;
    }

    //
    // First of all check the status for the SyncHead
    //
    ret = getSyncHeaderStatusCode(syncBody, &storeNextNonce);
    if ((ret <= 200) || (ret >=299)) {
        goto finally;
    }

finally:
    safeDelete(&syncBody);    
    return ret;
}


wchar_t* SyncMLDMProcessor::processMessage(wchar_t* msg, SyncMLDMBuilder *syncMLDMBuilder, int *cmdId, int msgRef, wchar_t* rootContext) {
    
    wchar_t* syncBody = NULL;
    wchar_t* cmdRef   = NULL;

    wchar_t* get      = NULL;
    wchar_t* exec     = NULL;
    wchar_t* replace  = NULL;    

    string getStatusMsg     = string();
    string execStatusMsg    = string();
    string replaceStatusMsg = string();    

    wchar_t* results          = NULL;
    wchar_t* responseStatusMsg= NULL;

    int ret          = 0;
    unsigned int pos = 0;
    unsigned int n   = 0;
    int size         = 0;

    syncBody = getElementContent(msg, TEXT("SyncBody"), NULL);
    if (syncBody == NULL) {
        lastErrorCode = ERR_REPRESENTATION;
        wsprintf(lastErrorMsg, TEXT("SyncBody not found!"));
        goto finally;
    }

    // Actually we don't implement Sequence command
    // we find every replace chunk
    
    //
    // process Replace command
    //
   
    do {
        safeDelete(&replace   );        

        replace = getElementContent(&syncBody[pos], TEXT("Replace"), &n);
  
        if (replace == NULL) {
            //
            // No more Item
            //
            break;
        }
        pos += n;
        
        ret = processReplaceCommand(replace, &cmdRef, syncMLDMBuilder, replaceStatusMsg, cmdId, msgRef, rootContext);

        // LOG.info (replaceStatusMsg);
       
    } while (replace != NULL);

        //
    // process Get command
    //
    pos = 0; n = 0; ret = 0;
    do {
        safeDelete(&get);
        safeDelete(&results);

        get = getElementContent(&syncBody[pos], TEXT("Get"), &n);

        if (get == NULL) {
            //
            // No more Item
            //
            break;
        }
        pos += n;

        ret = processGetCommand(get, &cmdRef, syncMLDMBuilder, getStatusMsg, cmdId, msgRef, rootContext);
         
        // LOG.info (getStatusMsg);

    } while (get != NULL);

    //
    // process Exec command
    //
    pos = 0; n = 0; ret = 0;
    do {
        safeDelete(&exec);

        exec = getElementContent(&syncBody[pos], TEXT("Exec"), &n);

        if (exec == NULL) {
            //
            // No more Item
            //
            break;
        }
        pos += n;

        ret = processExecCommand(exec, &cmdRef, syncMLDMBuilder, execStatusMsg, cmdId, msgRef, rootContext);

        // LOG.info (execStatusMsg);

    } while (exec != NULL);
    

    if (length(replaceStatusMsg) != 0)       
        size += wcslen(replaceStatusMsg); 
    if (length(getStatusMsg    ) != 0)
        size += wcslen(getStatusMsg    );
    if (length(execStatusMsg   ) != 0)
        size += wcslen(execStatusMsg   ); 
     
    
    if (size != 0) {
        responseStatusMsg = new wchar_t [size + 1];
        *responseStatusMsg = 0;
        if (length(replaceStatusMsg) != 0) 
            wcscat(responseStatusMsg, (wchar_t*)replaceStatusMsg);
        if (length(getStatusMsg    ) != 0)
            wcscat(responseStatusMsg, (wchar_t*)getStatusMsg    );
        if (length(execStatusMsg   ) != 0 )
            wcscat(responseStatusMsg, (wchar_t*)execStatusMsg   );
        
    }


finally:

    safeDelete(&syncBody);
    safeDelete(&get     );
    safeDelete(&exec    );
    safeDelete(&replace );
    safeDelete(&cmdRef  );   
    safeDelete(&results );
    
    return responseStatusMsg;
}



/****************************************************************/

/*
* return value for the last replace action.
* IN:  replace. It is chunk of message retrieved from server message. It can be that there are more Items in a Replace tag
* IN:  cmdRef . It id the command to refer this replace
* IN:  syncMLDMBuilder. It is the builder of the syncML message
* OUT:  is the variable that contains the syncML message to return to server
* IN:  is the global command id 
* IN:  msgRef.  It refers to server msgId
*/
int SyncMLDMProcessor::processReplaceCommand(wchar_t* replace, wchar_t** cmdRef, SyncMLDMBuilder* syncMLDMBuilder, 
                                             string &replaceStatusMsg, int *cmdId, int msgRef, wchar_t* rootContext) {
    

    int itemCount      = 0;
    unsigned int pos   = 0;
    unsigned int n,i,t = 0;
    int ret            = 0;
    
    wchar_t* item   = NULL;
    wchar_t* locUri = NULL;
    wchar_t* data   = NULL;    

    
    unsigned int dimArray = 4; // has to be number of possible ret status

    int     statusRet [4]; // is the same of dimArray
    int     isFull    [4];
    string  targetRef [4];
    
    statusRet[0] = VALUE_OK         ;   // 200 ok
    statusRet[1] = ERR_NOT_FOUND    ;   // 404 not found
    statusRet[2] = ERR_EQUALS_VALUES;   // 418 data already presents
    statusRet[3] = ERR_COMMAND_FAILED;  // 500 generic error

    for (i = 0; i < dimArray; i++)
        isFull[i] = 0;
    
    for (i = 0; i < dimArray; i++)
        targetRef[i] = string();
        
    BOOL onlyOneStatus = FALSE; 

    string uriData = string();

    (*cmdRef) = getElementContent(replace, TEXT("CmdID"), NULL);

    do {

        ++itemCount;

        safeDelete(&item  );
        safeDelete(&locUri);
        safeDelete(&data  );


        item = getElementContent(&replace[pos], TEXT("Item"), &n);
        if (item == NULL) {
            //
            // No more Item
            //
            break;
        }
        pos += n;

        locUri = getElementContent(item, TEXT("LocURI"), NULL);
        if (locUri == NULL) {
            //
            // Representation error
            //
            ret = lastErrorCode = ERR_REPRESENTATION;
            wsprintf(lastErrorMsg, TEXT("Missing Replace/Item/LocURI element!"));

            goto finally;
        }

        data = getElementContent(item, TEXT("Data"), NULL);
        
        // Make replace in Register Tree!!!! 

        ret = actionReplace (locUri, data, rootContext);       
        
        // in data di createTargetUriData we have to put ret of action replace
        // this function create only <targetUri, data> chunk   

        for (i = 0; i < dimArray; i++) {
            if (ret == statusRet[i])  {
                targetRef[i] += TEXT("<TargetRef>");
                targetRef[i] += locUri;
                targetRef[i] += TEXT("</TargetRef>\n"); 
                isFull[i] = 1;
            }
        }    

    } while (item != NULL);
    
    for (i = 0; i < dimArray; i++) {
        t = t + isFull[i];
    }
    if (t < 2)
        onlyOneStatus = TRUE;

    for (i = 0; i < dimArray; i++)
        replaceStatusMsg += syncMLDMBuilder->createReplaceStatusMessage(msgRef, *cmdRef, cmdId, targetRef[i], statusRet[i], onlyOneStatus);  
    

finally:
    safeDelete(&item    );
    safeDelete(&locUri  );
    safeDelete(&data    );

    return ret;

}

/*
* It use for process replace command. It create a managementNode and execute the replace action using data.
* It returns the status code for the action.
*/

int SyncMLDMProcessor::actionReplace(wchar_t* localUri, wchar_t* data, wchar_t* rootContext) {
    
    int ret       = 0;
    lastErrorCode = 0;

    wchar_t  property     [DIM_MANAGEMENT_PATH];
    wchar_t  pathWithDot  [DIM_MANAGEMENT_PATH];
    wchar_t  path         [DIM_MANAGEMENT_PATH];

    wmemset(path,           0, DIM_MANAGEMENT_PATH);
    wmemset(pathWithDot,    0, DIM_MANAGEMENT_PATH);
   
    ManagementNode *management = NULL;
    Win32ManagementTree managementTree;
    
    // in this case, the name after last '/' char represents the property name...
    getNodeName(localUri, property, DIM_MANAGEMENT_PATH);

    if (property == NULL || wcscmp(property, TEXT("")) == 0) {
        ret = ERR_COMMAND_FAILED;     // property node not found. It's something like <LocURI></LocURI>
        goto finally;
    }

    getNodeContext(localUri, pathWithDot, DIM_MANAGEMENT_PATH);    

    // create a path for getting children name
    if (wcscmp(pathWithDot, TEXT("")) == 0 || 
                wcscmp(pathWithDot, TEXT(".")) == 0)    // it's something like <LocURI>.</LocURI> or <LocURI>./DevInfo</LocURI> 
        
        wsprintf(path, TEXT("%s"), rootContext);
    else if (pathWithDot[0] == L'.' && pathWithDot[1] == L'/')      // it's something like <LocURI>./DevInfo/xxxx</LocURI>

        wsprintf(path, TEXT("%s/%s"), rootContext, &pathWithDot[2]);
    else                                                // it's something like <LocURI>DevInfo</LocURI> or <LocURI>../DevInfo</LocURI>

        wsprintf(path, TEXT("%s/%s"), rootContext, pathWithDot);

    managementTree  = Win32ManagementTree();   
    management      = managementTree.getManagementNode(path);

    management->setPropertyValue(property, data, FALSE, FALSE);
    
    
    if (lastErrorCode != 0) {
        if (lastErrorCode == ERR_EQUALS_VALUES) {
            ret = ERR_EQUALS_VALUES;    //418
            goto finally;
        }
        if (lastErrorCode == ERR_KEY_NOT_FOUND) {
            ret = ERR_NOT_FOUND;       // 404 node not found
            goto finally;
        }
        ret = ERR_COMMAND_FAILED;      // 500 generic error
        goto finally;
    }
    
    ret = VALUE_OK;
    
    
finally:   
    
    if (management != NULL)
        delete management;
    
    management = NULL;
    return ret;
}


/*
* return value for the replace action.
* IN:  exec. It is chunk of message retrieved from server message. It can be that there are more Items in a Replace tag
* IN: cmdRef.  It is the cmdRef to put into Status message
*/
int SyncMLDMProcessor::processExecCommand(wchar_t* exec, wchar_t** cmdRef, SyncMLDMBuilder* syncMLDMBuilder, 
                                             string &execStatusMsg, int *cmdId, int msgRef, wchar_t* rootContext) {
    

    int itemCount    = 0;
    unsigned int pos = 0;
    unsigned int n,i,t   = 0;
    int ret          = 0;

    wchar_t* item   = NULL;
    wchar_t* locUri = NULL;
    wchar_t* data   = NULL;

    unsigned int dimArray = 2;

    int     statusRet [2]; // is the same of dimArray
    int     isFull [2];
    string  targetRef[2];
    
    BOOL onlyOneStatus = FALSE; 

    statusRet[0] = VALUE_OK;        // 200
    statusRet[1] = ERR_NOT_FOUND;   // 404
    
    for (i = 0; i < dimArray; i++)
        isFull[i] = 0;        
    
    for (i = 0; i < dimArray; i++)
        targetRef[i] = string();    
    
    (*cmdRef) = getElementContent(exec, TEXT("CmdID"), NULL);

    do {

        ++itemCount;

        safeDelete(&item  );
        safeDelete(&locUri);
        safeDelete(&data  );


        item = getElementContent(&exec[pos], TEXT("Item"), &n);
        if (item == NULL) {
            //
            // No more Item
            //
            break;
        }
        pos += n;

        locUri = getElementContent(item, TEXT("LocURI"), NULL);
        if (locUri == NULL) {
            //
            // Representation error
            //
            ret = lastErrorCode = ERR_REPRESENTATION;
            wsprintf(lastErrorMsg, TEXT("Missing Replace/Item/LocURI element!"));

            goto finally;
        }

        data = getElementContent(item, TEXT("Data"), NULL);
        
        ret = actionExec (locUri, data);     

        // in data di createTargetUriData we have to put ret of action replace
        // this function create only <targetUri, data> chunk            
        
        for (i = 0; i < dimArray; i++) {
            if (ret == statusRet[i])  {
                targetRef[i] += TEXT("<TargetRef>");
                targetRef[i] += locUri;
                targetRef[i] += TEXT("</TargetRef>\n"); 
                isFull[i] = 1;
            }
        }            

    } while (item != NULL);

    for (i = 0; i < dimArray; i++) {
        t = t + isFull[i];
    }
    if (t < 2)
        onlyOneStatus = TRUE;

    for (i = 0; i < dimArray; i++)
        execStatusMsg += syncMLDMBuilder->createExecStatusMessage(msgRef, *cmdRef, cmdId, targetRef[i], statusRet[i], onlyOneStatus);  
  

finally:
    safeDelete(&item    );
    safeDelete(&locUri  );
    safeDelete(&data    );
    
    return ret;

}

/*
* To be implemented action to do...
*/
int SyncMLDMProcessor::actionExec(wchar_t* localUri, wchar_t* data) {

    wchar_t tmp[512];
    wsprintf(tmp, TEXT("---->actionExec method: localURI -> \"%s\"  Data -> \"%s\""), localUri, data);

    LOG.info(tmp);
    return VALUE_OK;
}

/*
* return value for the replace action.
* IN:  get. It is chunk of message retrieved from server message. It can be that there are more Items in a Replace tag
* IN: cmdRef.  It is the cmdRef to put into Status message
*/
int SyncMLDMProcessor::processGetCommand(wchar_t* get, wchar_t** cmdRef, SyncMLDMBuilder* syncMLDMBuilder, 
                                             string &getStatusMsg, int *cmdId, int msgRef, wchar_t* rootContext) {


    int statusCmd      = 0;
    unsigned int pos   = 0;
    unsigned int n,i,t = 0;
    int ret            = 0;
    unsigned int dimArray = 3;

    int     statusRet [3]; // is the same of dimArray
    int     isFull [3];
    string  targetRef[3];
    
    statusRet[0] = VALUE_OK;            // 200
    statusRet[1] = ERR_NOT_FOUND;       // 404
    statusRet[2] = ERR_COMMAND_FAILED;  // 500

    for (i = 0; i < dimArray; i++) 
        isFull[i] = 0;        
    
    for (i = 0; i < dimArray; i++)     
        targetRef[i] = string();
    
    
    BOOL onlyOneStatus = FALSE; 

    wchar_t* item   = NULL;
    wchar_t* locUri = NULL;
    
    string results = string();
    string uriData = string();
        
    (*cmdRef) = getElementContent(get, TEXT("CmdID"), NULL);
    
    do {

        safeDelete(&item  );
        safeDelete(&locUri);
        
        string itemResults = string();
        
        item = getElementContent(&get[pos], TEXT("Item"), &n);
        if (item == NULL) {
            //
            // No more Item
            //
            break;
        }
        pos += n;

        locUri = getElementContent(item, TEXT("LocURI"), NULL);
        if (locUri == NULL) {
            //
            // Representation error
            //
            ret = lastErrorCode = ERR_REPRESENTATION;
            wsprintf(lastErrorMsg, TEXT("Missing Replace/Item/LocURI element!"));

            goto finally;
        }
        
        // Make get in Registry Tree. 

        ret = actionGet (locUri, rootContext, syncMLDMBuilder, itemResults);       
        results += itemResults;

        for (i = 0; i < dimArray; i++) {
            if (ret == statusRet[i])  {
                targetRef[i] += TEXT("<TargetRef>");
                targetRef[i] += locUri;
                targetRef[i] += TEXT("</TargetRef>\n"); 
                isFull[i] = 1;
            }
        }    
        
    } while (item != NULL);
    
    for (i = 0; i < dimArray; i++) {
        t = t + isFull[i];
    }
    if (t < 2)
        onlyOneStatus = TRUE;

    for (i = 0; i < dimArray; i++)
        getStatusMsg += syncMLDMBuilder->createGetStatusMessage(msgRef, *cmdRef, cmdId, targetRef[i], statusRet[i], onlyOneStatus);  
    
    if (length(results) != 0) {
        syncMLDMBuilder->completeResults(results, msgRef, *cmdRef, ++(*cmdId));        
        getStatusMsg += results;
    }
        
finally:
    safeDelete(&item    );
    safeDelete(&locUri  );
    
    return ret;

}

int SyncMLDMProcessor::actionGet(wchar_t* localUri,  wchar_t* rootContext, SyncMLDMBuilder* syncMLDMBuilder, 
                                string &results) {
    
    int ret       = 0;
    lastErrorCode = 0;

    wchar_t  property     [DIM_MANAGEMENT_PATH];
    wchar_t  value        [DIM_MANAGEMENT_PATH];
    wchar_t  pathWithDot  [DIM_MANAGEMENT_PATH];
    wchar_t  path         [DIM_MANAGEMENT_PATH];

    wmemset(path,           0, DIM_MANAGEMENT_PATH);
    wmemset(property,       0, DIM_MANAGEMENT_PATH);     
    wmemset(value,          0, DIM_MANAGEMENT_PATH);    
    wmemset(pathWithDot,    0, DIM_MANAGEMENT_PATH);
    
       
    ManagementNode *management = NULL;
    Win32ManagementTree managementTree;
    
    // Suppose nodeName is a key
    getNodeName(localUri, property, DIM_MANAGEMENT_PATH);
    
    if (property == NULL || wcscmp(property, TEXT("")) == 0) {
        ret = ERR_COMMAND_FAILED;     // property node not found. It's something like <LocURI></LocURI>
        goto finally;
    }
    getNodeContext(localUri, pathWithDot, DIM_MANAGEMENT_PATH);
    
    // create a path for getting nodeResult
    if (wcscmp(pathWithDot, TEXT(""))  == 0)        // it's something like <LocURI>.</LocURI>
        wsprintf(path, TEXT("%s"), rootContext);
    else if (wcscmp(pathWithDot, TEXT(".")) == 0)   // it's something like <LocURI>./DevInfo</LocURI>       
        wsprintf(path, TEXT("%s/%s"), rootContext, property);    
    else if (pathWithDot[0] == L'.')                // it's something like <LocURI>./DevInfo/xxxx</LocURI>
        wsprintf(path, TEXT("%s/%s/%s"), rootContext, &pathWithDot[2], property);
    else                                            // it's something like <LocURI>DevInfo</LocURI>
        wsprintf(path, TEXT("%s/%s/%s"), rootContext, pathWithDot, property);
    
    
    managementTree = Win32ManagementTree();   
    management = managementTree.getManagementNode(path);
        
    management->getChildrenName(path, value, DIM_MANAGEMENT_PATH, FALSE);
        
    if (lastErrorCode == 0) { // found that property is a key!! 
        results = syncMLDMBuilder->createResultsMessage(localUri, value, TRUE);

    }
    else {
        // have to try to get property value. Reset errorCode and ErrorMsg
        lastErrorCode = 0;
        wsprintf(lastErrorMsg, TEXT(""));
        delete management;

        wsprintf(pathWithDot, path);
        wmemset(path, 0, DIM_MANAGEMENT_PATH);

        getNodeContext(pathWithDot, path, DIM_MANAGEMENT_PATH);
        management = managementTree.getManagementNode(path);

        int err = management->getPropertyValue(property, value, DIM_MANAGEMENT_PATH, FALSE);

        if (lastErrorCode != 0 || err != 0) { // found that property is not a value!! 
            if (lastErrorCode == ERR_INVALID_CONTEXT || err == ERR_KEY_NOT_FOUND) // error
                ret = ERR_NOT_FOUND;
            else
                ret = ERR_COMMAND_FAILED;
            goto finally;
        }

        results = syncMLDMBuilder->createResultsMessage(localUri, value, FALSE);
       
    }
    
    ret = VALUE_OK;

finally:

    if (management != NULL)
        delete management;
    
    management = NULL;
    return ret;
}


/*
 * Returns the SyncHeader/RespURI element of the given message. If the element is not
 * found it returns NULL. The returned respURI is allocated with the new operator
 * and must be discarded with delete by the caller.
 *
 * @param msg - the message - NOT NULL
 */
wchar_t* SyncMLDMProcessor::getRespURI(wchar_t* msg) {
    wchar_t* header  = NULL;
    wchar_t* respURI = NULL;

    header = getElementContent(msg, TEXT("SyncHdr"), NULL);
    if (header == NULL) {
        goto finally;
    }

    respURI = getElementContent(header, TEXT("RespURI"), NULL);

    finally:

    //safeDelete(&header);

    return respURI;
}


// ------------------------------------------------------------- Private methods





/*
 * Extracts the content of a tag into a SyncML message. It is supposed the
 * message is a valid SyncML message. It returns NULL in case the tag is not
 * found or the XML fragment is not in the expected form.
 * The returned pointer (if not NULL) is allocated with the new operator and
 * must be discarded with the operator delete.
 *
 * @param xml the xml fragment
 * @param tag the tag we want the content
 * @param pos (OUTPUT) the position just after the closing tag (ignored if NULL)
 *
 */
wchar_t* SyncMLDMProcessor::getElementContent(wchar_t* xml, wchar_t* tag, unsigned int* pos) {
    wchar_t* p1       = NULL;
    wchar_t* p2       = NULL;
    wchar_t* ret      = NULL;

    wchar_t openTag[50];
    wchar_t closeTag[50];

    unsigned int l = wcslen(tag);

    if (pos != NULL) {
        *pos = 0;
    }

    wsprintf(openTag, TEXT("<%s>"), tag);
    wsprintf(closeTag, TEXT("</%s>"), tag);

    p1 = wcsstr(xml, openTag);

    if (p1 == NULL) {
        //
        // This is abcxyz
        //
        goto finally;
    }
    p1 = p1+l+2;

    if (*p1 == 0) {
        //
        // This is abc<tag>\0
        //
        goto finally;
    }

    p2 = wcsstr(p1, closeTag);

    if (p2 == NULL) {
        //
        // This is abc<tag>xyz\0
        //
        goto finally;
    }

    ret = new wchar_t[p2-p1+1];

    wcsncpy(ret, p1, p2-p1);
    ret[p2-p1] = 0;

    if (pos != NULL) {
        *pos = p2-xml+l+3;
    }

    finally:

    return ret;

}

/*
 * Returns the status code for the SyncHeader command included
 * in the message sent by the client.
 *
 * @param syncBody - the SyncBody content
 */
int SyncMLDMProcessor::getSyncHeaderStatusCode(wchar_t* syncBody, wchar_t** storeNextNonce) {
    wchar_t* status     = NULL;
    wchar_t* cmdRef     = NULL;
    wchar_t* data       = NULL;
    wchar_t* chal       = NULL;
    wchar_t* authType   = NULL;

    int ret = -1;

    //
    // It must be the first one
    //
    status = getElementContent(syncBody, TEXT("Status"), NULL);
    if (status == NULL) {
        //
        // It should not happen
        //
        wsprintf(lastErrorMsg, TEXT("Status command for SyncHeader not found!"));
        lastErrorCode = ERR_REPRESENTATION;
        goto finally;
    }

    cmdRef = getElementContent(status, TEXT("CmdRef"), NULL);
    if ((cmdRef == NULL) || (wcscmp(cmdRef, TEXT("0")))) {
        //
        // It should not happen
        //
        wsprintf(lastErrorMsg, TEXT("Status/CmdRef or not found or not referring to SyncHeader!"));
        lastErrorCode = ERR_REPRESENTATION;
        goto finally;
    }

    data = getElementContent(status, TEXT("Data"), NULL);
    if (data == NULL) {
        //
        // It should not happen
        //
        wsprintf(lastErrorMsg, TEXT("Status/Data not found!"));
        lastErrorCode = ERR_REPRESENTATION;
        goto finally;
    }

    ret = wcstoul(data, NULL, 10);
    
    // get NextNonce and store in storeNextNonce variable...
    chal = getElementContent(status, TEXT("Chal"), NULL);
    if (chal != NULL) {
        authType = getElementContent(chal, TEXT("Type"), NULL);
        
        //
        // if authtype != syncml:auth-md5 <NextNonce> has not to be present. 
        // So Auth has to be performed only one time. return NextNonce NULL 
        //
        if (authType != NULL && wcscmp(authType, TEXT("syncml:auth-md5")) == 0)                
            *storeNextNonce = getElementContent(chal, TEXT("NextNonce"), NULL);

    }


    finally:

    safeDelete(&status   );
    safeDelete(&cmdRef   );
    safeDelete(&data     );
    safeDelete(&chal     );

    return ret;
}


/*
 * Returns the status code for the Alert relative to the given source.
 *
 * @param syncBody - the SyncBody content
 * @param sourceName - the name of the source
*/
int SyncMLDMProcessor::getAlertAndReplaceStatusCode(wchar_t* syncBody) {
    unsigned int pos = 0, prev = 0;

    wchar_t* status    = NULL;
    wchar_t* cmd       = NULL;
    wchar_t* data      = NULL;
    wchar_t* sourceRef = NULL;

    int ret = 0;

    do {
        safeDelete(&status   );
        safeDelete(&cmd      );
        safeDelete(&sourceRef);
        
        status = getElementContent(&syncBody[prev], TEXT("Status"), &pos);
        prev = prev + pos;

        if (status == NULL) {
            break;
        }

        cmd = getElementContent(status, TEXT("Cmd"), NULL);
        if (cmd == NULL) {
            //
            // It should not happen
            //
            wsprintf(lastErrorMsg, TEXT("Status/Cmd not found!"));
            lastErrorCode = ERR_REPRESENTATION;
            goto finally;
        }

        if (wcscmp(TEXT("Alert"), cmd) == 0) {
            data = getElementContent(status, TEXT("Data"), NULL);
            if ((data != NULL) && (wcscmp(data, TEXT("200")) == 0)) {
                ret = wcstoul(data, NULL, 10);
                continue;
            }
            else {
                ret = -1;
                break;
            }
        }

        if (wcscmp(TEXT("Replace"), cmd) == 0) {
            data = getElementContent(status, TEXT("Data"), NULL);
            if ((data != NULL) && (wcscmp(data, TEXT("200")) == 0)) {
                ret = wcstoul(data, NULL, 10);
                continue;
            }
            else {
                ret = -1;
                break;
            }
        }

    } while(status != NULL);

    


    finally:

    safeDelete(&status   );
    safeDelete(&cmd      );
    safeDelete(&data     );
    safeDelete(&sourceRef);

    return ret;
}
