/*
 * Copyright (C) 2003-2006 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef INCL_SYNCMLDMMANAGER_DEVICE_MANAGER
    #define INCL_SYNCMLDMMANAGER_DEVICE_MANAGER

    #include "spdm/common/Constants.h"
    #include "spdm/common/DeviceManager.h"
    #include "spdm/win32/DevInfo.h"
    #include "spdm/win32/DevDetail.h"
    #include "spdm/win32/SyncMLDMConfig.h"
    #include "spdm/win32/SyncMLDMBuilder.h"
    #include "http/common/TransportAgent.h"
    #include "spdm/win32/SyncMLDMProcessor.h"



    class __declspec(dllexport) SyncMLDMManager : public DeviceManager  {

    public:
        
        /*
         * Constructor.
         */
        SyncMLDMManager();
        SyncMLDMManager(wchar_t* accNodeName);
        ~SyncMLDMManager();
       
        /*
         * Returns the root management node for the DeviceManager.
         *
         * The ManagementNode is created with the new operator and must be
         * discarded by the caller with the operator delete.
         */
        ManagementNode* getRootManagementNode();

        /*
         * Returns the management node identified by the given node pathname
         * (relative to the root management node). If the node is not found
         * NULL is returned; additional info on the error condition can be
         * retrieved calling getLastError() and getLastErrorMessage()
         *
         * The ManagementNode is created with the new operator and must be
         * discarded by the caller with the operator delete.
         */
        ManagementNode* getManagementNode(const wchar_t* node);

       /*
        * Returns a message about the management of a DM session. In this function there is all the 
        * things to complete a management phase.
        */        
        int startSession(wchar_t* DMSessionId, BOOL isClientNotified);
        
        /*--------------------------------------------------------------*/

        /*
         * These object are used for complete message and configuration
         */

        SyncMLDMConfig*     syncMLDMConfig;
        DevInfo*            devInfo;
        DevDetail*      devDetail;
        SyncMLDMBuilder*    syncMLDMBuilder;
        TransportAgent*     transportAgent;
        SyncMLDMProcessor*  syncMLDMProcessor;

    };

#endif