/*
 * Copyright (C) 2003-2006 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef INCL_SYNCMLDM_PROCESSOR
    #define INCL_SYNCMLDM_PROCESSOR

    
    #include "http/common/Constants.h"
    #include "spds/common/Constants.h"    
    #include "ptypes/ptypes.h"
    #include "spdm/common/Constants.h"
    #include "spdm/win32/SyncMLDMBuilder.h"
   

    USING_PTYPES


    /*
     * This class is responsible for the processing of the incoming messages.
     */

    class __declspec(dllexport) SyncMLDMProcessor {

    private:
        /*
         * Extracts the content of a tag into a SyncML message. It is supposed the
         * message is a valid SyncML message. It returns NULL in case the tag is not
         * found or the XML fragment is not in the expected form.
         * The returned pointer (if not NULL) is allocated with the new operator and
         * must be discarded with the operator delete.
         *
         * @param xml the xml fragment
         * @param tag the tag we want the content
         * @param pos (OUTPUT) the position where the tag is found (ignored if NULL)
         *
         */
        wchar_t* SyncMLDMProcessor::getElementContent(wchar_t* xml, wchar_t* tag, unsigned int* pos);

        /*
         * Returns the status code for the SyncHeader command included
         * in the message sent by the client.
         *
         * @param syncBody - the SyncBody content
         */
        int SyncMLDMProcessor::getSyncHeaderStatusCode(wchar_t* syncBody, wchar_t** storeNextNonce);        

        /*
         * Returns the status code for the Alert relative to the given source.
         *
         * @param syncBody - the SyncBody content
         * @param sourceName - the name of the source
         */
        int getAlertAndReplaceStatusCode(wchar_t* syncBody);


    public:

        /*
         * Constructor
         */
        SyncMLDMProcessor();

        /*
         * Processes the initialization response. Returns 0 in case of success, an
         * error code in case of error.
         *
         * @param msg the response from the server
         */
        int processInitResponse(wchar_t* msg, wchar_t** storeNextNonce);

        /*
         * Processes the synchronization response. Returns 0 in case of success, an
         * error code in case of error.
         * It feeds the given source with the server side modifications
         *
         * @param msg the response from the server
         */
        int processCommandResponse(wchar_t* msg);

        /*
         * Returns the SyncHeader/RespURI element of the given message. If the element is not
         * found it returns NULL. The returned respURI is allocated with the new operator
         * and must be discarded with delete by the caller.
         *
         * @param msg - the message - NOT NULL
         */
        wchar_t* getRespURI(wchar_t* msg);
        
        int getServerMessageId  (wchar_t* msg);
        int processHeader       (wchar_t* msg);
        
        /*
         * Used in DMManager to process the server message after authentication. It retrives and process
         * Replace, Exec and Get command. It also build the message about status and Result. This returned
         * message will be complete by SyncMLDMBuilder->createResponseStatusMessage(...)
         */
        wchar_t* processMessage (wchar_t* msg, SyncMLDMBuilder *syncMLDMBuilder, int *cmdId, int msgRef, wchar_t* rootContext);
        

        /*
         * Process response chunk with Replace command
         */
        int processReplaceCommand(wchar_t* replace, wchar_t** cmdRef, SyncMLDMBuilder* syncMLDMBuilder, 
                            string &locUriData, int* cmdId, int msgRef, wchar_t* rootContext);

        /*
         * Process response chunk with Get command
         */
        int processGetCommand(wchar_t* get, wchar_t** cmdRef, SyncMLDMBuilder* syncMLDMBuilder, 
                            string &locUriData, int* cmdId, int msgRef, wchar_t* rootContext);

         /*
         * Process response chunk with Exec command
         */
        int processExecCommand(wchar_t* exec, wchar_t** cmdRef, SyncMLDMBuilder* syncMLDMBuilder, 
                                             string &execStatusMsg, int *cmdId, int msgRef, wchar_t* rootContext); 


        int actionReplace(wchar_t* localUri, wchar_t* data, wchar_t* rootContext);
        
        int actionGet(wchar_t* localUri, wchar_t* rootContext, SyncMLDMBuilder* syncMLDMBuilder,        
                string &results);

        int actionExec(wchar_t* localUri, wchar_t* data);
    };

#endif